@@ -14,6 +14,7 @@
   limitations under the License.
 
   Authors: Wang Zhulei (wangzhulei@sogou-inc.com)
+           Xie Han (xiehan@sogou-inc.com)
 */
 
 #include <assert.h>
@@ -45,8 +46,7 @@ static KafkaCgroup __create_cgroup(const KafkaCgroup *c)
 
 /**********Client**********/
 
-class __ComplexKafkaTask : public WFComplexClientTask<KafkaRequest, KafkaResponse,
-													  struct __ComplexKafkaTaskCtx>
+class __ComplexKafkaTask : public WFComplexClientTask<KafkaRequest, KafkaResponse, int>
 {
 public:
 	__ComplexKafkaTask(int retry_max, __kafka_callback_t&& callback) :
@@ -54,7 +54,7 @@ public:
 	{
 		is_user_request_ = true;
 		is_redirect_ = false;
-		ctx_.kafka_error = 0;
+		ctx_ = 0;
 	}
 
 protected:
@@ -117,7 +117,6 @@ private:
 	virtual int keep_alive_timeout();
 	virtual int first_timeout();
 	bool has_next();
-	bool check_redirect();
 	bool process_produce();
 	bool process_fetch();
 	bool process_metadata();
@@ -310,19 +309,17 @@ CommMessageIn *__ComplexKafkaTask::message_in()
 
 bool __ComplexKafkaTask::init_success()
 {
-	TransportType type = TT_TCP;
-	if (uri_.scheme)
+	enum TransportType type;
+
+	if (uri_.scheme && strcasecmp(uri_.scheme, "kafka") == 0)
+		type = TT_TCP;
+	else if (uri_.scheme && strcasecmp(uri_.scheme, "kafkas") == 0)
+		type = TT_TCP_SSL;
+	else
 	{
-		if (strcasecmp(uri_.scheme, "kafka") == 0)
-			type = TT_TCP;
-		//else if (uri_.scheme && strcasecmp(uri_.scheme, "kafkas") == 0)
-		//	type = TT_TCP_SSL;
-		else
-		{
-			this->state = WFT_STATE_TASK_ERROR;
-			this->error = WFT_ERR_URI_SCHEME_INVALID;
-			return false;
-		}
+		this->state = WFT_STATE_TASK_ERROR;
+		this->error = WFT_ERR_URI_SCHEME_INVALID;
+		return false;
 	}
 
 	std::string username, password, sasl, client;
@@ -367,7 +364,6 @@ bool __ComplexKafkaTask::init_success()
 	}
 
 	this->WFComplexClientTask::set_transport_type(type);
-
 	return true;
 }
 
@@ -409,52 +405,11 @@ int __ComplexKafkaTask::first_timeout()
 	return ret + KAFKA_ROUNDTRIP_TIMEOUT;
 }
 
-bool __ComplexKafkaTask::check_redirect()
-{
-	struct sockaddr_storage addr;
-	socklen_t addrlen = sizeof addr;
-	const struct sockaddr *paddr = (const struct sockaddr *)&addr;
-	KafkaBroker *coordinator = this->get_req()->get_cgroup()->get_coordinator();
-
-	//always success
-	this->get_peer_addr((struct sockaddr *)&addr, &addrlen);
-	if (!coordinator->is_equal(paddr, addrlen))
-	{
-		if (coordinator->is_to_addr())
-		{
-			const struct sockaddr *addr_coord;
-			socklen_t addrlen_coord;
-
-			coordinator->get_broker_addr(&addr_coord, &addrlen_coord);
-			set_redirect(TT_TCP, addr_coord, addrlen_coord,
-						 this->WFComplexClientTask::info_);
-		}
-		else
-		{
-			std::string url = "kafka://";
-			url += user_info_ + "@";
-			url += coordinator->get_host();
-			url += ":" + std::to_string(coordinator->get_port());
-
-			ParsedURI uri;
-			URIParser::parse(url, uri);
-			set_redirect(std::move(uri));
-		}
-
-		return true;
-	}
-	else
-	{
-		this->init(TT_TCP, paddr, addrlen, this->WFComplexClientTask::info_);
-		return false;
-	}
-}
-
 bool __ComplexKafkaTask::process_find_coordinator()
 {
 	KafkaCgroup *cgroup = this->get_resp()->get_cgroup();
-	ctx_.kafka_error = cgroup->get_error();
-	if (ctx_.kafka_error)
+	ctx_ = cgroup->get_error();
+	if (ctx_)
 	{
 		this->error = WFT_ERR_KAFKA_CGROUP_FAILED;
 		this->state = WFT_STATE_TASK_ERROR;
@@ -463,8 +418,18 @@ bool __ComplexKafkaTask::process_find_coordinator()
 	else
 	{
 		this->get_req()->set_cgroup(*cgroup);
-		is_redirect_ = check_redirect();
+		KafkaBroker *coordinator = cgroup->get_coordinator();
+		std::string url(uri_.scheme);
+		url += "://";
+		url += user_info_ + "@";
+		url += coordinator->get_host();
+		url += ":" + std::to_string(coordinator->get_port());
+
+		ParsedURI uri;
+		URIParser::parse(url, uri);
+		set_redirect(std::move(uri));
 		this->get_req()->set_api_type(Kafka_JoinGroup);
+		is_redirect_ = true;
 		return true;
 	}
 }
@@ -472,16 +437,6 @@ bool __ComplexKafkaTask::process_find_coordinator()
 bool __ComplexKafkaTask::process_join_group()
 {
 	KafkaResponse *msg = this->get_resp();
-	if (!msg->get_cgroup()->get_coordinator()->is_to_addr())
-	{
-		struct sockaddr_storage addr;
-		socklen_t addrlen = sizeof addr;
-		const struct sockaddr *paddr = (const struct sockaddr *)&addr;
-		this->get_peer_addr((struct sockaddr *)&addr, &addrlen);
-		msg->get_cgroup()->get_coordinator()->set_broker_addr(paddr, addrlen);
-		msg->get_cgroup()->get_coordinator()->set_to_addr(1);
-	}
-
 	switch(msg->get_cgroup()->get_error())
 	{
 	case KAFKA_MEMBER_ID_REQUIRED:
@@ -498,7 +453,7 @@ bool __ComplexKafkaTask::process_join_group()
 		break;
 
 	default:
-		ctx_.kafka_error = msg->get_cgroup()->get_error();
+		ctx_ = msg->get_cgroup()->get_error();
 		this->error = WFT_ERR_KAFKA_CGROUP_FAILED;
 		this->state = WFT_STATE_TASK_ERROR;
 		return false;
@@ -509,8 +464,8 @@ bool __ComplexKafkaTask::process_join_group()
 
 bool __ComplexKafkaTask::process_sync_group()
 {
-	ctx_.kafka_error = this->get_resp()->get_cgroup()->get_error();
-	if (ctx_.kafka_error)
+	ctx_ = this->get_resp()->get_cgroup()->get_error();
+	if (ctx_)
 	{
 		this->error = WFT_ERR_KAFKA_CGROUP_FAILED;
 		this->state = WFT_STATE_TASK_ERROR;
@@ -538,7 +493,7 @@ bool __ComplexKafkaTask::process_metadata()
 		case 0:
 			break;
 		default:
-			ctx_.kafka_error = meta->get_error();
+			ctx_ = meta->get_error();
 			this->error = WFT_ERR_KAFKA_META_FAILED;
 			this->state = WFT_STATE_TASK_ERROR;
 			return false;
@@ -598,7 +553,7 @@ bool __ComplexKafkaTask::process_fetch()
 		case KAFKA_OFFSET_OUT_OF_RANGE:
 			break;
 		default:
-			ctx_.kafka_error = toppar->get_error();
+			ctx_ = toppar->get_error();
 			this->error = WFT_ERR_KAFKA_FETCH_FAILED;
 			this->state = WFT_STATE_TASK_ERROR;
 			return false;
@@ -658,8 +613,8 @@ bool __ComplexKafkaTask::process_produce()
 
 bool __ComplexKafkaTask::process_sasl_handshake()
 {
-	ctx_.kafka_error = this->get_resp()->get_broker()->get_error();
-	if (ctx_.kafka_error)
+	ctx_ = this->get_resp()->get_broker()->get_error();
+	if (ctx_)
 	{
 		this->error = WFT_ERR_KAFKA_SASL_DISALLOWED;
 		this->state = WFT_STATE_TASK_ERROR;
@@ -670,8 +625,8 @@ bool __ComplexKafkaTask::process_sasl_handshake()
 
 bool __ComplexKafkaTask::process_sasl_authenticate()
 {
-	ctx_.kafka_error = this->get_resp()->get_broker()->get_error();
-	if (ctx_.kafka_error)
+	ctx_ = this->get_resp()->get_broker()->get_error();
+	if (ctx_)
 	{
 		this->error = WFT_ERR_KAFKA_SASL_DISALLOWED;
 		this->state = WFT_STATE_TASK_ERROR;
@@ -681,18 +636,6 @@ bool __ComplexKafkaTask::process_sasl_authenticate()
 
 bool __ComplexKafkaTask::has_next()
 {
-	struct sockaddr_storage addr;
-	socklen_t addrlen = sizeof addr;
-	//always success
-	this->get_peer_addr((struct sockaddr *)&addr, &addrlen);
-
-	const struct sockaddr *paddr = (const struct sockaddr *)&addr;
-	if (!this->get_resp()->get_broker()->is_to_addr())
-	{
-		this->get_resp()->get_broker()->set_broker_addr(paddr, addrlen);
-		this->get_resp()->get_broker()->set_to_addr(1);
-	}
-
 	switch (this->get_resp()->get_api_type())
 	{
 	case Kafka_Produce:
@@ -718,8 +661,8 @@ bool __ComplexKafkaTask::has_next()
 	case Kafka_LeaveGroup:
 	case Kafka_DescribeGroups:
 	case Kafka_Heartbeat:
-		ctx_.kafka_error = this->get_resp()->get_cgroup()->get_error();
-		if (ctx_.kafka_error)
+		ctx_ = this->get_resp()->get_cgroup()->get_error();
+		if (ctx_)
 		{
 			this->error = WFT_ERR_KAFKA_CGROUP_FAILED;
 			this->state = WFT_STATE_TASK_ERROR;
@@ -772,14 +715,10 @@ bool __ComplexKafkaTask::finish_once()
 	}
 	else
 	{
-		this->disable_retry();
 		this->get_resp()->set_api_type(this->get_req()->get_api_type());
 		this->get_resp()->set_api_version(this->get_req()->get_api_version());
 	}
 
-	if (ctx_.cb)
-		ctx_.cb(this);
-
 	return true;
 }
 
@@ -809,20 +748,8 @@ __WFKafkaTask *__WFKafkaTaskFactory::create_kafka_task(const ParsedURI& uri,
 	return task;
 }
 
-__WFKafkaTask *__WFKafkaTaskFactory::create_kafka_task(const struct sockaddr *addr,
-													   socklen_t addrlen,
-													   const std::string& info,
-													   int retry_max,
-													   __kafka_callback_t callback)
-{
-	auto *task = new __ComplexKafkaTask(retry_max, std::move(callback));
-
-	task->init(TT_TCP, addr, addrlen, info);
-	task->set_keep_alive(KAFKA_KEEPALIVE_DEFAULT);
-	return task;
-}
-
-__WFKafkaTask *__WFKafkaTaskFactory::create_kafka_task(const char *host,
+__WFKafkaTask *__WFKafkaTaskFactory::create_kafka_task(enum TransportType type,
+													   const char *host,
 													   unsigned short port,
 													   const std::string& info,
 													   int retry_max,
@@ -830,10 +757,10 @@ __WFKafkaTask *__WFKafkaTaskFactory::create_kafka_task(const char *host,
 {
 	auto *task = new __ComplexKafkaTask(retry_max, std::move(callback));
 
-	std::string url = "kafka://";
+	std::string url = (type == TT_TCP_SSL ? "kafkas://" : "kafka://");
 
 	if (!info.empty())
-		url += info;
+		url += info + "@";
 
 	url += host;
 	url += ":" + std::to_string(port);
